package org.test;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.lang3.time.DateUtils;
import org.drools.core.ClockType;
import org.drools.core.time.impl.PseudoClockScheduler;
import org.jbpm.workflow.instance.WorkflowRuntimeException;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.kie.api.KieBase;
import org.kie.api.KieServices;
import org.kie.api.event.rule.DefaultAgendaEventListener;
import org.kie.api.event.rule.RuleFlowGroupActivatedEvent;
import org.kie.api.runtime.KieRuntime;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.conf.ClockTypeOption;
import org.kie.internal.builder.conf.RuleEngineOption;

public class TimeOperatorTest {

	protected static final KieBase KIE_BASE;
	private SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
	private KieSession kSession;
	
	static {
		System.setProperty(RuleEngineOption.PROPERTY_NAME, RuleEngineOption.PHREAK.name());
		System.setProperty(ClockTypeOption.PROPERTY_NAME, ClockType.PSEUDO_CLOCK.getId());
		System.setProperty("drools.dateformat", "dd/MM/yyyy HH:mm");
		KIE_BASE = KieServices.Factory.get().getKieClasspathContainer().getKieBase();
	}
	
	@Test
	public void testTimeOperator_ko() throws InterruptedException, IOException {
		OtherSimpleFact fact = new OtherSimpleFact("code1");
		fact.setDateEvt(DateUtils.addDays(new Date(), -50).getTime());
		kSession.insert(fact);
		try {
			kSession.startProcess("Sample");
			kSession.fireAllRules();
		} catch (WorkflowRuntimeException e) {
			e.printStackTrace();	
			Assert.assertNull(e);
		}
	}

	@Before
	public void initSession() {
		kSession = KIE_BASE.newKieSession();
		
		if(!KIE_BASE.getProcesses().isEmpty()) {
			kSession.addEventListener(new DefaultAgendaEventListener() {
				@Override
				public void afterRuleFlowGroupActivated(RuleFlowGroupActivatedEvent event) {
					KieRuntime kieRuntime = event.getKieRuntime();
					if (kieRuntime instanceof KieSession) {
						((KieSession) kieRuntime).fireAllRules();
					}
				}
			});
		}
		long currentTime = System.currentTimeMillis();
		PseudoClockScheduler clock = kSession.getSessionClock();
		clock.setStartupTime(currentTime);
		System.out.println("Pseudo clock is set to " + sdf.format(new Date(currentTime)));
	}
	
}

